; ========================================================================================================-
; Chipmunk2D SDK by Lunasole [2015/11]
; ==========================================================================================================================-
; Thanks to IceSoft, author of Purebasic bindings to Chipmunk v5.3.5.
; Without using some of his tricks&tips me with my current one-month PB experience would get much more annoying problems doing this stuff ^^
; ============================================================================================================================================-
; Chipmunk2D lib version: 	7.0.1
; PureBasic version: 		5.40
; SDK version:				0.0.36
; ============================-
EnableExplicit
; WARNING: the int usage can be problematic if compile PB app To x64. In this case int size = 8 bytes and that will ruin many structures, etc. Should be Long type used instead, or also prepare x64 version of ChipMunk
; WARNING: there is Align = #CP_STRUCTURE_ALIGN everywhere, for some better perfomance but more memory consumption. You have to use the same align with all your custom structures which will interact with chipmunk structures, else it can lead to data corruption if copying memory directly [PB default align = 1]. To change align value ChipMunk library must be recompiled.

; ========================================================================================================
;	Chipmunk2D Pseudotypes
; ========================================================================================================
; Chipmunk :: Basic Types. The macro here used to represent them for handy usage

Macro cpFloat			: d : EndMacro ; double.		Depends on chipmunk build: double if ifndef CP_USE_DOUBLES, else it uses regular Float
Macro cpBool 			: a : EndMacro ; int [??].		Like every good C library that wants good cross language compatibility, you end up defining your own boolean type. :-\ Defaults to int.

Macro cpCollisionType 	: l : EndMacro ; unsigned int.	Unique identifier for collision shape types. Defaults to unsigned int. Defined type must support the == operator.
Macro cpGroup 			: l : EndMacro ; unsigned int.	Unique identifier for collision groups. Defaults to unsigned int. A CP_NO_GROUP value is defined that can be used when you dont want to specify a group. Defined type must support the equality == operator.
Macro cpBitmask 		: l : EndMacro ; unsigned int.	Type used for collision filter categories and masks. Defaults to unsigned int. Defined type must support the bitwise AND & operator.

; Some others pseudotypes
Macro cpCollisionID 	: l : EndMacro ; 
Macro cpTimestamp 		: l : EndMacro ; unsigned int

Macro cpHashValue 		: l : EndMacro

; Enums. The macro here are used as pseutotypes which can take values of corresponding enum
Macro cpShapeType 		: l : EndMacro
	Enumeration cpShapeType
		#CP_CIRCLE_SHAPE
		#CP_SEGMENT_SHAPE
		#CP_POLY_SHAPE
		#CP_NUM_SHAPES
	EndEnumeration
	
Macro cpArbiterState 	: l : EndMacro
	Enumeration cpArbiterState
		; Arbiter is active And its the first collision.
		#CP_ARBITER_STATE_FIRST_COLLISION
		; Arbiter is active And its Not the first collision.
		#CP_ARBITER_STATE_NORMAL
		; Collision has been explicitly ignored.
		; Either by returning false from a begin collision handler Or calling cpArbiterIgnore().
		#CP_ARBITER_STATE_IGNORE
		; Collison is no longer active. A space will cache an arbiter For up To cpSpace.collisionPersistence more steps.
		#CP_ARBITER_STATE_CACHED
		; Collison arbiter is invalid because one of the shapes was removed.
		#CP_ARBITER_STATE_INVALIDATED
	EndEnumeration

Macro cpBodyType 		: l : EndMacro
	Enumeration cpBodyType
		; A dynamic body is one that is affected by gravity, forces, And collisions.
		; This is the Default body type.
		#CP_BODY_TYPE_DYNAMIC
		; A kinematic body is an infinite mass, user controlled body that is Not affected by gravity, forces Or collisions.
		; Instead the body only moves based on it's velocity.
		; Dynamic bodies collide normally With kinematic bodies, though the kinematic body will be unaffected.
		; Collisions between two kinematic bodies, Or a kinematic body And a Static body produce collision callbacks, but no collision response.
		#CP_BODY_TYPE_KINEMATIC
		; A Static body is a body that never (Or rarely) moves. If you move a Static body, you must call one of the cpSpaceReindex*() functions.
		; Chipmunk uses this information To optimize the collision detection.
		; Static bodies do Not produce collision callbacks when colliding With other Static bodies.
		#CP_BODY_TYPE_STATIC
	EndEnumeration

Macro cpSpaceDebugDrawFlags: l : EndMacro
	Enumeration  cpSpaceDebugDrawFlags
		#CP_SPACE_DEBUG_DRAW_SHAPES = 1<<0
		#CP_SPACE_DEBUG_DRAW_CONSTRAINTS = 1<<1
		#CP_SPACE_DEBUG_DRAW_COLLISION_POINTS = 1<<2
	EndEnumeration


; ========================================================================================================
;	Initiation constants
; ========================================================================================================

	#CP_STRUCTURE_ALIGN				= 4 ; structure align, should be same as chipmunk library uses
	#CP_MAX_CONTACTS_PER_ARBITER 	= 2	; Size of the Array of contact points
	#CP_POLY_SHAPE_INLINE_ALLOC 	= 6 ; Size of cpPolyShape\_planes array (const * 2)

; ========================================================================================================
;	Structures
; ========================================================================================================

	Structure cpArray Align #CP_STRUCTURE_ALIGN
		num.l ; int
		max.l
	 	*arr.Long
	EndStructure
	
	Structure cpVect Align #CP_STRUCTURE_ALIGN ; Chipmunk's 2D vector type.
		X.cpFloat
		Y.cpFloat
	EndStructure
	
	Structure cpTransform Align #CP_STRUCTURE_ALIGN
		a.cpFloat
		b.cpFloat
		c.cpFloat
		d.cpFloat
		tx.cpFloat
		ty.cpFloat
	EndStructure
	
	Structure cpSpatialIndexClass Align #CP_STRUCTURE_ALIGN
		*cpSpatialIndexDestroyImpl ; destroy
		
		*cpSpatialIndexCountImpl ; count
		*cpSpatialIndexEachImpl ; each
		
		*cpSpatialIndexContainsImpl ; contains
		*cpSpatialIndexInsertImpl ; insert
		*cpSpatialIndexRemoveImpl ; remove
		
		*cpSpatialIndexReindexImpl ; reindex
		*cpSpatialIndexReindexObjectImpl ; reindexObject
		*cpSpatialIndexReindexQueryImpl ; reindexQuery
		
		*cpSpatialIndexQueryImpl ; query
		*cpSpatialIndexSegmentQueryImpl ; segmentQuery
	EndStructure
	
	Structure cpSpatialIndex Align #CP_STRUCTURE_ALIGN
		*klass.cpSpatialIndexClass 
		
		*cpSpatialIndexBBFunc ;  bbfunc
		
		*dynamicIndex.cpSpatialIndex 
		*staticIndex.cpSpatialIndex 
	EndStructure
	
	Structure cpBB  Align #CP_STRUCTURE_ALIGN ; Chipmunk's axis-aligned 2D bounding box type. (left, bottom, right, top)
		L.cpFloat
		B.cpFloat 
		R.cpFloat 
		T.cpFloat
	EndStructure
	
	Structure cpBBTree Align #CP_STRUCTURE_ALIGN
		spatialIndex.cpSpatialIndex
		*cpBBTreeVelocityFunc ;velocityFunc
		
		*leaves.cpHashSet
		*root.node
		
		*pooledNodes.node
		*pooledPairs.pair
		*allocatedBuffers.cpArray
		stamp.cpTimestamp
	EndStructure
	
	
	Structure cpMat2x2 Align #CP_STRUCTURE_ALIGN ; Row major [[a, b][c d]]
		a.cpFloat : b.cpFloat : c.cpFloat : d.cpFloat 
	EndStructure
	
	Structure cpSplittingPlane Align #CP_STRUCTURE_ALIGN
		v0.cpVect : n.cpVect
	EndStructure
	
	Structure cpHashSetBin Align #CP_STRUCTURE_ALIGN
		*elt
		hash.cpHashValue
		*next.cpHashSetBin
	EndStructure
	
	Structure cpHashSet Align #CP_STRUCTURE_ALIGN
		entries.l : size.l ; unsigned int
		
		*cpHashSetEqlFunc ; eql.
		*default_value
		
		*table.cpHashSetBin
		*pooledBins.cpHashSetBin
		
		*allocatedBuffers.cpArray
	EndStructure
	
	
	Structure Bounds Align #CP_STRUCTURE_ALIGN
		min.cpFloat : max.cpFloat
	EndStructure
	
	Structure TableCell Align #CP_STRUCTURE_ALIGN
		*obj
		bounds.Bounds
	EndStructure
	
	Structure cpSweep1D Align #CP_STRUCTURE_ALIGN
		spatialIndex.cpSpatialIndex
		
		num.l; int
		max.l; int
		*table.TableCell
	EndStructure

; ========================================================================================================
; Rigid Bodies: A rigid body holds the physical properties of an object. (mass, position, rotation, velocity, etc.)
; It does not have a shape until you attach one or more collision shapes to it. If youve done physics with particles before,
; rigid bodies differ in that they are able to rotate. Rigid bodies generally tend to have a 1:1 correlation to sprites in a game.
; You should structure your game so that you use the position and rotation of the rigid body for drawing your sprite.
; ========================================================================================================
	
	Structure sleeping Align #CP_STRUCTURE_ALIGN; Extends cpBody
		*root.cpBody
		*next.cpBody
		idleTime.cpFloat
	EndStructure
	
	Structure cpBody Align #CP_STRUCTURE_ALIGN
		; Integration functions
		*cpBodyVelocityFunc; velocity_func.
		*cpBodyPositionFunc; position_func.
		
		; mass And it's inverse
		mass.cpFloat
		mass_inv.cpFloat
		
		; moment of inertia And it's inverse
		inertia.cpFloat
		inertia_inv.cpFloat
		
		; center of gravity
		centerOfGravity.cpVect
		
		; position, velocity, force
		position.cpVect
		velocity.cpVect
		force.cpVect
		
		; Angle, angular velocity, torque (radians)
		angle.cpFloat
		angle_vel.cpFloat
		torque.cpFloat
		
		transform.cpTransform
		
		*userData;.cpDataPointer
		
		; "pseudo-velocities" used For eliminating overlap.
		; Erin Catto has some papers that talk about what these are.
		v_bias.cpVect
		w_bias.cpFloat
		
		*space.cpSpace
		
		*shapeList.cpShape
		*arbiterList.cpArbiter
		*constraintList.cpConstraint
		
		sleeping.sleeping
	EndStructure
	
; ========================================================================================================
; Collision Shapes: By attaching shapes to bodies, you can define the a bodys shape. 
; You can attach as many shapes to a single body as you need to in order to define a complex shape. 
; Shapes contain the surface properties of an object such as how much friction or elasticity it has.
; ========================================================================================================

	Structure cpShapeFilter  Align #CP_STRUCTURE_ALIGN; Fast collision filtering type that is used To determine If two objects collide before calling collision Or query callbacks.
		; Two objects With the same non-zero group value do Not collide.
		; This is generally used To group objects in a composite object together To disable self collisions.
		group.cpGroup
		; A bitmask of user definable categories that this object belongs To.
		; The category/mask combinations of both objects in a collision must agree For a collision To occur.
		categories.cpBitmask
		; A bitmask of user definable category types that this object object collides With.
		; The category/mask combinations of both objects in a collision must agree For a collision To occur.
		mask.cpBitmask
	EndStructure
	
	Structure cpShapeMassInfo Align #CP_STRUCTURE_ALIGN
		 m.cpFloat
		 i.cpFloat
		 cog.cpVect
		 area.cpFloat
	EndStructure
	
	Structure cpShapeClass Align #CP_STRUCTURE_ALIGN
		type.cpShapeType
		
		*cpShapeCacheDataImpl 	; cacheData
		*cpShapeDestroyImpl 	; destroy
		*cpShapePointQueryImpl 	; pointQuery
		*cpShapeSegmentQueryImpl ; *segmentQuery
	EndStructure
	
	Structure cpShape Align #CP_STRUCTURE_ALIGN
		*klass.cpShapeClass
		
		*space.cpSpace
		*body.cpBody
		massInfo.cpShapeMassInfo
		bb.cpBB
		
		sensor.cpBool
		
		e.cpFloat
		u.cpFloat
		surfaceV.cpVect
	
		*userData;cpDataPointer
		
		type.cpCollisionType
		filter.cpShapeFilter
		
		*next.cpShape
		*prev.cpShape
		
		hashid.cpHashValue
	EndStructure
	
	Structure cpPolyShape Align #CP_STRUCTURE_ALIGN
		shape.cpShape
		
		r.cpFloat
		
		count.l ; int
		; The untransformed planes are appended at the End of the transformed planes.
		*planes.cpSplittingPlane
		
		; Allocate a small number of splitting planes internally For simple poly.
		_planes.cpSplittingPlane [2 * #CP_POLY_SHAPE_INLINE_ALLOC]
	EndStructure
	
	Structure cpCircleShape Align #CP_STRUCTURE_ALIGN
		shape.cpShape
		
		c.cpVect : tc.cpVect
		r.cpFloat
	EndStructure
	
	Structure cpSegmentShape Align #CP_STRUCTURE_ALIGN
		shape.cpShape
		
		a.cpVect : b.cpVect : n.cpVect
		ta.cpVect : tb.cpVect : tn.cpVect
		r.cpFloat
		
		a_tangent.cpVect : b_tangent.cpVect
	EndStructure
	
	Structure cpCollisionHandler Align #CP_STRUCTURE_ALIGN; Struct that holds function callback pointers To configure custom collision handling. Collision handlers have a pair of types; when a collision occurs between two shapes that have these types, the collision handler functions are triggered.
		; Collision type identifier of the first shape that this handler recognizes.
		; In the collision handler callback, the shape With this type will be the first argument. Read only.
		typeA.cpCollisionType
		; Collision type identifier of the second shape that this handler recognizes.
		; In the collision handler callback, the shape With this type will be the second argument. Read only.
		typeB.cpCollisionType
		; This function is called when two shapes With types that match this collision handler begin colliding.
		*cpCollisionBeginFunc; beginFunc;
		; This function is called each Step when two shapes With types that match this collision handler are colliding.
		; It's called before the collision solver runs so that you can affect a collision's outcome.
		*cpCollisionPreSolveFunc; preSolveFunc;
		; This function is called each Step when two shapes With types that match this collision handler are colliding.
		; It's called after the collision solver runs so that you can read back information about the collision to trigger events in your game.
		*cpCollisionPostSolveFunc ;postSolveFunc;
		; This function is called when two shapes With types that match this collision handler stop colliding.
		*cpCollisionSeparateFunc; separateFunc;
		; This is a user definable context pointer that is passed To all of the collision handler functions.
		*userData ;cpDataPointer
	EndStructure
	
	Structure cpCollisionInfo Align #CP_STRUCTURE_ALIGN
		*a.cpShape
		*b.cpShape
		id.cpCollisionID
		
		n.cpVect
		
		count.l ; int
		*arr.cpContact
	EndStructure

; ========================================================================================================
; Constraints/Joints: Constraints and joints describe how bodies are attached to each other.
; ========================================================================================================

	Structure cpConstraint Align #CP_STRUCTURE_ALIGN
		*klass.cpConstraintClass
		
		*space.cpSpace
		
		*a.cpBody
		*b.cpBody
		*next_a.cpConstraint
		*next_b.cpConstraint
		
		maxForce.cpFloat
		errorBias.cpFloat
		maxBias.cpFloat
		
		collideBodies.cpBool
		
		*cpConstraintPreSolveFunc ; preSolve
		*cpConstraintPostSolveFunc ; postSolve
		
		*userData ;.cpDataPointer
	EndStructure
	
	Structure cpConstraintClass Align #CP_STRUCTURE_ALIGN
		*cpConstraintPreStepImpl; preStep;
		*cpConstraintApplyCachedImpulseImpl ; applyCachedImpulse;
		*cpConstraintApplyImpulseImpl ;applyImpulse;
		*cpConstraintGetImpulseImpl ; getImpulse;
	EndStructure
	
	Structure cpGrooveJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		grv_n.cpVect : grv_a.cpVect : grv_b.cpVect
		anchorB.cpVect
		
		grv_tn.cpVect
		clamp.cpFloat
		r1.cpVect : r2.cpVect
		k.cpMat2x2
		
		jAcc.cpVect
		bias.cpVect
	EndStructure
	
	Structure cpPinJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		dist.cpFloat
		
		r1.cpVect : r2.cpVect
		n.cpVect
		nMass.cpFloat
		
		jnAcc.cpFloat
		bias.cpFloat
	EndStructure
	
	Structure cpPivotJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		
		r1.cpVect : r2.cpVect
		k.cpMat2x2
		
		jAcc.cpVect
		bias.cpVect
	EndStructure
	
	Structure cpRotaryLimitJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		min.cpFloat : max.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpRatchetJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		angle.cpFloat : phase.cpFloat : ratchet.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpGearJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		phase.cpFloat : ratio.cpFloat
		ratio_inv.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpSlideJoint Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		min.cpFloat : max.cpFloat
		
		r1.cpVect : r2.cpVect
		n.cpVect
		nMass.cpFloat
		
		jnAcc.cpFloat
		bias.cpFloat
	EndStructure
	
	Structure cpSimpleMotor Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		rate.cpFloat
		
		iSum.cpFloat
		
		jAcc.cpFloat
	EndStructure

; ========================================================================================================
; Spaces: Spaces are containers for simulating objects in Chipmunk.
; You add bodies, shapes and joints to a space and then update the space as a whole.
; They control how all the rigid bodies, shapes, and constraints interact together.
; ========================================================================================================

	Structure cpSpace Align #CP_STRUCTURE_ALIGN
		iterations.l ; int
		
		gravity.cpVect
		damping.cpFloat 
		
		idleSpeedThreshold.cpFloat
		sleepTimeThreshold.cpFloat
		
		collisionSlop.cpFloat
		collisionBias.cpFloat
		collisionPersistence.cpTimestamp
		
		*userData ;cpDataPointer
		
		stamp.cpTimestamp
		curr_dt.cpFloat
		
		*dynamicBodies.cpArray
		*staticBodies.cpArray
		*rousedBodies.cpArray
		*sleepingComponents.cpArray
		
		shapeIDCounter.cpHashValue
		*staticShapes.cpSpatialIndex
		*dynamicShapes.cpSpatialIndex
		
		*constraints.cpArray
		
		*arbiters.cpArray
		*contactBuffersHead.cpContactBufferHeader
		*cachedArbiters.cpHashSet
		*pooledArbiters.cpArray
		
		*allocatedBuffers.cpArray
		locked.l ; unsigned int 
		
		usesWildcards.cpBool
		*collisionHandlers.cpHashSet
		defaultHandler.cpCollisionHandler
		
		skipPostStep.cpBool
		*postStepCallbacks.cpArray
		
		*staticBody.cpBody
		_staticBody.cpBody
	EndStructure
	
	Structure cpSpaceDebugColor Align #CP_STRUCTURE_ALIGN ; Color type To use With the space Debug drawing API.
		r.float : g.float : b.float : a.float
	EndStructure
	
	Structure cpSpaceDebugDrawOptions Align #CP_STRUCTURE_ALIGN ; Struct used With cpSpaceDebugDraw() containing drawing callbacks And other drawing settings.
		; Function that will be invoked To draw circles.
		*cpSpaceDebugDrawCircleImpl ; drawCircle.
		; Function that will be invoked To draw line segments.
		*cpSpaceDebugDrawSegmentImpl ; drawSegment.
		; Function that will be invoked To draw thick line segments.
		*cpSpaceDebugDrawFatSegmentImpl ; drawFatSegment.
		; Function that will be invoked To draw convex polygons.
		*cpSpaceDebugDrawPolygonImpl ; drawPolygon.
		; Function that will be invoked To draw dots.
		*cpSpaceDebugDrawDotImpl ; drawDot.
		
		; Flags that request which things To draw (collision shapes, constraints, contact points).
		flags.cpSpaceDebugDrawFlags
		; Outline color passed To the drawing function.
		shapeOutlineColor.cpSpaceDebugColor
		; Function that decides what fill color To draw shapes using.
		*cpSpaceDebugDrawColorForShapeImpl ; colorForShape.
		; Color passed To drawing functions For constraints.
		constraintColor.cpSpaceDebugColor
		; Color passed To drawing functions For collision points.
		collisionPointColor.cpSpaceDebugColor
		
		; User defined context pointer passed To all of the callback functions As the 'data' argument.
		*Data ; cpDataPointer
	EndStructure
	
	Structure cpSpaceHash Align #CP_STRUCTURE_ALIGN
		spatialIndex.cpSpatialIndex
		
		numcells.l ; int
		celldim.cpFloat
		
		*table.cpSpaceHashBin
		*handleSet.cpHashSet
		
		*pooledBins.cpSpaceHashBin
		*pooledHandles.cpArray
		*allocatedBuffers.cpArray
		
		stamp.cpTimestamp
	EndStructure

; ========================================================================================================
; Other structures [unclassified yet]
; ========================================================================================================

	Structure cpArbiterThread Align #CP_STRUCTURE_ALIGN
		*next.cpArbiter
		*prev.cpArbiter
	EndStructure
	
	Structure cpArbiter Align #CP_STRUCTURE_ALIGN
		e.cpFloat
		u.cpFloat
		surface_vr.cpVect
		
		*Data ; .cpDataPointer
		
		*a.cpShape
		*b.cpShape
		
		*body_a.cpBody 
		*body_b.cpBody
		thread_a.cpArbiterThread 
		thread_b.cpArbiterThread
		
		count.l ; int
		*contacts.cpContact
		n.cpVect 
		
	; 	// Regular, wildcard A And wildcard B collision handlers.
		*handler.cpCollisionHandler 
		*handlerA.cpCollisionHandler 
		*handlerB.cpCollisionHandler
		swapped.cpBool
		
		stamp.cpTimestamp
		state.cpArbiterState
	EndStructure
	
	
	Structure cpContactPoints Align #CP_STRUCTURE_ALIGN
		; The position of the contact on the surface of each shape.
		pointA.cpVect
		pointB.cpVect
		; Penetration distance of the two shapes. Overlapping means it will be negative.
		; This value is calculated As cpvdot(cpvsub(point2, point1), normal) And is ignored by cpArbiterSetContactPointSet().
		distance.cpFloat
	EndStructure
	
	Structure cpContactPointSet Align #CP_STRUCTURE_ALIGN ; A struct that wraps up the important collision Data For an arbiter.
		; The number of contact points in the set.
		count.l; int
		
		; The normal of the collision.
		normal.cpVect
		
		Points.cpContactPoints [#CP_MAX_CONTACTS_PER_ARBITER]
	EndStructure
	
	Structure cpContact Align #CP_STRUCTURE_ALIGN
		r1.cpVect
		r2.cpVect
		
		nMass.cpFloat
		tMass.cpFloat
		bounce.cpFloat
		
		jnAcc.cpFloat
		jtAcc.cpFloat
		jBias.cpFloat
		
		bias.cpFloat
		
		hash.cpHashValue
	EndStructure
	
	
	Structure cpDampedSpring Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		restLength.cpFloat
		stiffness.cpFloat
		damping.cpFloat
		*cpDampedSpringForceFunc ; springForceFunc
		
		target_vrn.cpFloat
		v_coef.cpFloat
		
		r1.cpVect : r2.cpVect
		nMass.cpFloat
		n.cpVect
		
		jAcc.cpFloat
	EndStructure
	
	Structure cpDampedRotarySpring Align #CP_STRUCTURE_ALIGN
		constraint.cpConstraint
		restAngle.cpFloat
		stiffness.cpFloat
		damping.cpFloat
		*cpDampedRotarySpringTorqueFunc ;springTorqueFunc
		
		target_wrn.cpFloat
		w_coef.cpFloat
		
		iSum.cpFloat
		jAcc.cpFloat
	EndStructure
	
	
	Structure cpPointQueryInfo Align #CP_STRUCTURE_ALIGN ; Point query info struct.
		; The nearest shape, NULL If no shape was within range.
		*shape.cpShape
		; The closest point on the shape's surface. (in world space coordinates)
		point.cpVect
		; The distance To the point. The distance is negative If the point is inside the shape.
		distance.cpFloat
		; The gradient of the signed distance function.
		; The value should be similar To info.p/info.d, but accurate even For very small values of info.d.
		gradient.cpVect
	EndStructure
	
	Structure cpSegmentQueryInfo Align #CP_STRUCTURE_ALIGN ; Segment query info struct.
		; The shape that was hit, Or NULL If no collision occured.
		*shape.cpShape
		; The point of impact.
		point.cpVect
		; The normal of the surface hit.
		normal.cpVect
		; The normalized distance along the query segment in the range [0, 1].
		alpha.cpFloat
	EndStructure



; ========================================================================================================
;	Chipmunk constants
; ========================================================================================================

	Global cpvzero.cpVect ; PB doesn't allow use structure as constant, so just declare it as global var
	

; ========================================================================================================
;	Useful routines
; ========================================================================================================
; TODO: the cpv[...] functions are inner functions of chipmunk so I didn't ported them when parsing files. They are doing some operations on vectors (like cpvadd: cpVect1 + cvVect2), this can be easily done without them, but maybe sometime I'll include them too. Currently they are rewriten here.

	Procedure cpv(*cpVect.cpVect, x.cpFloat, y.cpFloat) ; Convenience constructor For cpVect structs.
		*cpVect\X = X
		*cpVect\Y = Y
	EndProcedure


; ========================================================================================================
;	Import section [auto-generated import list]
; ========================================================================================================
; Some functions here have "_pb" extension. They are pointed to modified version of original functions,
; cause those originals returning structures by value which PB doesn't support (and which generally is a bad practice).
; 	Those functions should be called like this [it is anyway better than use SizeOf[] field in every structure]:
; 		Define RetStructure
; 		someFunction_pb (RetStructure) ; the RetStructure here will receive function result
; 	.. instead of:
;		Define RetStructure = someFunction ()
;
;	The original functions are excluded from import list here, but they are still present in .lib file, so if you want you can 
;	add them too and use some ASM hacks to make them work with PB.
;	----------------------------------------------------------------------------------------------------------------------------
; 	UPD: there is also another case when this extension added: if function has "_pb" ext, but there is no *lp_out argument,
;		then it's return was not modified but some arguments were structures byval [which PB doesn't supports too] 
;		and they were changed to pointers, functions in such cases were renamed just to know about it's modification
; ========================================================================================================

; TODO: fix cpMessage paramarray argument representation (or damn it, this function seems totally useless, let it be commented)
; TODO: make also builds with SSE/SSE2 support? not sure it will give much better perfomance (need more info about how it works), also using SSE2 is not wise cause lot of older AMD processors have no support of it and unable to execute such files
ImportC ".\lib\chipmunk\chipmunk.7.0.1_x86.lib"
	cpArbiterApplyCachedImpulse   (*arb.cpArbiter, dt_coef.cpFloat) As "?cpArbiterApplyCachedImpulse@@YAXPAUcpArbiter@@N@Z"
	cpArbiterApplyImpulse   (*arb.cpArbiter) As "?cpArbiterApplyImpulse@@YAXPAUcpArbiter@@@Z"
	cpArbiterCallWildcardBeginA.cpBool  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardBeginA"
	cpArbiterCallWildcardBeginB.cpBool  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardBeginB"
	cpArbiterCallWildcardPostSolveA   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPostSolveA"
	cpArbiterCallWildcardPostSolveB   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPostSolveB"
	cpArbiterCallWildcardPreSolveA.cpBool   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPreSolveA"
	cpArbiterCallWildcardPreSolveB.cpBool   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPreSolveB"
	cpArbiterCallWildcardSeparateA  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardSeparateA"
	cpArbiterCallWildcardSeparateB  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardSeparateB"
	cpArbiterGetBodies  (*arb.cpArbiter, *a.cpBody, *b.cpBody) As "_cpArbiterGetBodies"
	cpArbiterGetContactPointSet_pb  (*arb.cpArbiter, *lp_out.cpContactPointSet) As "?cpArbiterGetContactPointSet_pb@@YAXPBUcpArbiter@@PAUcpContactPointSet@@@Z"
	cpArbiterGetCount.l   (*arb.cpArbiter) As "_cpArbiterGetCount"
	cpArbiterGetDepth.cpFloat   (*arb.cpArbiter, i.l) As "_cpArbiterGetDepth"
	cpArbiterGetFriction.cpFloat  (*arb.cpArbiter) As "_cpArbiterGetFriction"
	cpArbiterGetNormal_pb   (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetNormal_pb@@YAXPBUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterGetPointA_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointA_pb@@YAXPBUcpArbiter@@HPAUcpVect@@@Z"
	cpArbiterGetPointB_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointB_pb@@YAXPBUcpArbiter@@HPAUcpVect@@@Z"
	cpArbiterGetRestitution.cpFloat   (*arb.cpArbiter) As "_cpArbiterGetRestitution"
	cpArbiterGetShapes  (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "_cpArbiterGetShapes"
	cpArbiterGetSurfaceVelocity_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetSurfaceVelocity_pb@@YAXPAUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterGetUserData.l  (*arb.cpArbiter) As "_cpArbiterGetUserData";  = *cpDataPointer
	cpArbiterIgnore.cpBool  (*arb.cpArbiter) As "_cpArbiterIgnore"
	cpArbiterInit.l   (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "?cpArbiterInit@@YAPAUcpArbiter@@PAU1@PAUcpShape@@1@Z";  = *cpArbiter
	cpArbiterIsFirstContact.cpBool  (*arb.cpArbiter) As "_cpArbiterIsFirstContact"
	cpArbiterIsRemoval.cpBool   (*arb.cpArbiter) As "_cpArbiterIsRemoval"
	cpArbiterPreStep  (*arb.cpArbiter, dt.cpFloat, slop.cpFloat, bias.cpFloat) As "?cpArbiterPreStep@@YAXPAUcpArbiter@@NNN@Z"
	cpArbiterSetContactPointSet   (*arb.cpArbiter, *set.cpContactPointSet) As "_cpArbiterSetContactPointSet"
	cpArbiterSetFriction  (*arb.cpArbiter, friction.cpFloat) As "_cpArbiterSetFriction"
	cpArbiterSetRestitution   (*arb.cpArbiter, restitution.cpFloat) As "_cpArbiterSetRestitution"
	cpArbiterSetSurfaceVelocity_pb  (*arb.cpArbiter, *vr.cpVect) As "?cpArbiterSetSurfaceVelocity_pb@@YAXPAUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterSetUserData  (*arb.cpArbiter, *cpDataPointer) As "_cpArbiterSetUserData"
	cpArbiterTotalImpulse_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterTotalImpulse_pb@@YAXPBUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterTotalKE.cpFloat  (*arb.cpArbiter) As "_cpArbiterTotalKE"
	cpArbiterUnthread   (*arb.cpArbiter) As "?cpArbiterUnthread@@YAXPAUcpArbiter@@@Z"
	cpArbiterUpdate   (*arb.cpArbiter, *info.cpCollisionInfo, *space.cpSpace) As "?cpArbiterUpdate@@YAXPAUcpArbiter@@PAUcpCollisionInfo@@PAUcpSpace@@@Z"

	cpAreaForCircle.cpFloat   (r1.cpFloat, r2.cpFloat) As "_cpAreaForCircle"
	cpAreaForPoly.cpFloat   (count.l, *verts.cpVect, r.cpFloat) As "_cpAreaForPoly"
	cpAreaForSegment_pb.cpFloat   (*a.cpVect, *b.cpVect, r.cpFloat) As "?cpAreaForSegment_pb@@YANPAUcpVect@@0N@Z"

	cpArrayContains.cpBool  (*arr.cpArray, *ptr) As "?cpArrayContains@@YAEPAUcpArray@@PAX@Z"
	cpArrayDeleteObj  (*arr.cpArray, *obj) As "?cpArrayDeleteObj@@YAXPAUcpArray@@PAX@Z"
	cpArrayFree   (*arr.cpArray) As "?cpArrayFree@@YAXPAUcpArray@@@Z"
	cpArrayFreeEach   (*arr.cpArray, *freeFunc) As "?cpArrayFreeEach@@YAXPAUcpArray@@P6AXPAX@Z@Z"
	cpArrayNew.l  (size.l) As "?cpArrayNew@@YAPAUcpArray@@H@Z";  = *cpArray
	cpArrayPop.l  (*arr.cpArray) As "?cpArrayPop@@YAPAXPAUcpArray@@@Z";  = *void
	cpArrayPush   (*arr.cpArray, *object) As "?cpArrayPush@@YAXPAUcpArray@@PAX@Z"

	cpBBTreeAlloc.l   () As "_cpBBTreeAlloc";  = *cpBBTree
	cpBBTreeInit.l  (*tree.cpBBTree, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpBBTreeInit";  = *cpSpatialIndex
	cpBBTreeNew.l   (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpBBTreeNew";  = *cpSpatialIndex
	cpBBTreeOptimize  (*index.cpSpatialIndex) As "_cpBBTreeOptimize"
	cpBBTreeSetVelocityFunc   (*index.cpSpatialIndex, *cpBBTreeVelocityFunc) As "_cpBBTreeSetVelocityFunc"

	cpBodyAccumulateMassFromShapes  (*body.cpBody) As "?cpBodyAccumulateMassFromShapes@@YAXPAUcpBody@@@Z"
	cpBodyActivate  (*body.cpBody) As "_cpBodyActivate"
	cpBodyActivateStatic  (*body.cpBody, *filter.cpShape) As "_cpBodyActivateStatic"
	cpBodyAddShape  (*body.cpBody, *shape.cpShape) As "?cpBodyAddShape@@YAXPAUcpBody@@PAUcpShape@@@Z"
	cpBodyAlloc.l   () As "_cpBodyAlloc";  = *cpBody
	cpBodyApplyForceAtLocalPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtLocalPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyForceAtWorldPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtWorldPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyImpulseAtLocalPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtLocalPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyImpulseAtWorldPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtWorldPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyDestroy   (*body.cpBody) As "_cpBodyDestroy"
	cpBodyEachArbiter   (*body.cpBody, *cpBodyArbiterIteratorFunc, *data) As "_cpBodyEachArbiter"
	cpBodyEachConstraint  (*body.cpBody, *cpBodyConstraintIteratorFunc, *data) As "_cpBodyEachConstraint"
	cpBodyEachShape   (*body.cpBody, *cpBodyShapeIteratorFunc, *data) As "_cpBodyEachShape"
	cpBodyFree  (*body.cpBody) As "_cpBodyFree"
	cpBodyGetAngle.cpFloat  (*body.cpBody) As "_cpBodyGetAngle"
	cpBodyGetAngularVelocity.cpFloat  (*body.cpBody) As "_cpBodyGetAngularVelocity"
	cpBodyGetCenterOfGravity_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetCenterOfGravity_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetForce_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetForce_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetMass.cpFloat   (*body.cpBody) As "_cpBodyGetMass"
	cpBodyGetMoment.cpFloat   (*body.cpBody) As "_cpBodyGetMoment"
	cpBodyGetPosition_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetPosition_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetRotation_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetRotation_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetSpace.l  (*body.cpBody) As "_cpBodyGetSpace";  = *cpSpace
	cpBodyGetTorque.cpFloat   (*body.cpBody) As "_cpBodyGetTorque"
	cpBodyGetType.cpBodyType  (*body.cpBody) As "_cpBodyGetType"
	cpBodyGetUserData.l   (*body.cpBody) As "_cpBodyGetUserData";  = *cpDataPointer
	cpBodyGetVelocity_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetVelocity_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetVelocityAtLocalPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtLocalPoint_pb@@YAXPBUcpBody@@PAUcpVect@@1@Z"
	cpBodyGetVelocityAtWorldPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtWorldPoint_pb@@YAXPBUcpBody@@PAUcpVect@@1@Z"
	cpBodyInit.l  (*body.cpBody, mass.cpFloat, moment.cpFloat) As "_cpBodyInit";  = *cpBody
	cpBodyIsSleeping.cpBool   (*body.cpBody) As "_cpBodyIsSleeping"
	cpBodyKineticEnergy.cpFloat   (*body.cpBody) As "_cpBodyKineticEnergy"
	cpBodyLocalToWorld_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyLocalToWorld_pb@@YAXPBUcpBody@@PBUcpVect@@PAU2@@Z"
	cpBodyNew.l   (mass.cpFloat, moment.cpFloat) As "_cpBodyNew";  = *cpBody
	cpBodyNewKinematic.l  () As "_cpBodyNewKinematic";  = *cpBody
	cpBodyNewStatic.l   () As "_cpBodyNewStatic";  = *cpBody
	cpBodyRemoveConstraint  (*body.cpBody, *constraint.cpConstraint) As "?cpBodyRemoveConstraint@@YAXPAUcpBody@@PAUcpConstraint@@@Z"
	cpBodyRemoveShape   (*body.cpBody, *shape.cpShape) As "?cpBodyRemoveShape@@YAXPAUcpBody@@PAUcpShape@@@Z"
	cpBodySetAngle  (*body.cpBody, angle.cpFloat) As "_cpBodySetAngle"
	cpBodySetAngularVelocity  (*body.cpBody, angularVelocity.cpFloat) As "_cpBodySetAngularVelocity"
	cpBodySetCenterOfGravity_pb   (*body.cpBody, *cog.cpVect) As "?cpBodySetCenterOfGravity_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetForce_pb   (*body.cpBody, *force.cpVect) As "?cpBodySetForce_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetMass   (*body.cpBody, mass.cpFloat) As "_cpBodySetMass"
	cpBodySetMoment   (*body.cpBody, moment.cpFloat) As "_cpBodySetMoment"
	cpBodySetPosition_pb  (*body.cpBody, *position.cpVect) As "?cpBodySetPosition_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetPositionUpdateFunc   (*body.cpBody, *cpBodyPositionFunc) As "_cpBodySetPositionUpdateFunc"
	cpBodySetTorque   (*body.cpBody, torque.cpFloat) As "_cpBodySetTorque"
	cpBodySetType   (*body.cpBody, type.cpBodyType) As "_cpBodySetType"
	cpBodySetUserData   (*body.cpBody, *cpDataPointer) As "_cpBodySetUserData"
	cpBodySetVelocity_pb  (*body.cpBody, *velocity.cpVect) As "?cpBodySetVelocity_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetVelocityUpdateFunc   (*body.cpBody, *cpBodyVelocityFunc) As "_cpBodySetVelocityUpdateFunc"
	cpBodySleep   (*body.cpBody) As "_cpBodySleep"
	cpBodySleepWithGroup  (*body.cpBody, *group.cpBody) As "_cpBodySleepWithGroup"
	cpBodyUpdatePosition  (*body.cpBody, dt.cpFloat) As "_cpBodyUpdatePosition"
	cpBodyUpdateVelocity_pb   (*body.cpBody, *gravity.cpVect, damping.cpFloat, dt.cpFloat) As "?cpBodyUpdateVelocity_pb@@YAXPAUcpBody@@PAUcpVect@@NN@Z"
	cpBodyWorldToLocal_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyWorldToLocal_pb@@YAXPBUcpBody@@PBUcpVect@@PAU2@@Z"

	cpBoxShapeInit.l  (*poly.cpPolyShape, *body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "_cpBoxShapeInit";  = *cpPolyShape
	cpBoxShapeInit2.l   (*poly.cpPolyShape, *body.cpBody, *cpBB, radius.cpFloat) As "_cpBoxShapeInit2";  = *cpPolyShape
	cpBoxShapeNew.l   (*body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "_cpBoxShapeNew";  = *cpShape
	cpBoxShapeNew2.l  (*body.cpBody, *cpBB, radius.cpFloat) As "_cpBoxShapeNew2";  = *cpShape

	cpCentroidForPoly_pb  (count.l, *verts.cpVect, *lp_out.cpVect) As "?cpCentroidForPoly_pb@@YAXHPBUcpVect@@PAU1@@Z"
	cpCheckSignedArea_pb.cpBool   (*a.cpVect, *b.cpVect, *c.cpVect) As "?cpCheckSignedArea_pb@@YAEPBUcpVect@@00@Z"

	cpCircleShapeAlloc.l  () As "_cpCircleShapeAlloc";  = *cpCircleShape
	cpCircleShapeGetOffset_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpCircleShapeGetOffset_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpCircleShapeGetRadius.cpFloat  (*shape.cpShape) As "_cpCircleShapeGetRadius"
	cpCircleShapeInit_pb.l  (*circle.cpCircleShape, *body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeInit_pb@@YAPAUcpCircleShape@@PAU1@PAUcpBody@@NPAUcpVect@@@Z";  = *cpCircleShape
	cpCircleShapeNew_pb.l   (*body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@NPAUcpVect@@@Z";  = *cpShape
	cpCircleShapeSetOffset_pb   (*shape.cpShape, *offset.cpVect) As "?cpCircleShapeSetOffset_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpCircleShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "_cpCircleShapeSetRadius"

	cpCollide_pb  (*a.cpShape, *b.cpShape, id.cpCollisionID, *contacts.cpContact, *lp_out.cpCollisionInfo) As "?cpCollide_pb@@YAXPBUcpShape@@0IPAUcpContact@@PAUcpCollisionInfo@@@Z"

	cpConstraintDestroy   (*constraint.cpConstraint) As "_cpConstraintDestroy"
	cpConstraintFree  (*constraint.cpConstraint) As "_cpConstraintFree"
	cpConstraintGetBodyA.l  (*constraint.cpConstraint) As "_cpConstraintGetBodyA";  = *cpBody
	cpConstraintGetBodyB.l  (*constraint.cpConstraint) As "_cpConstraintGetBodyB";  = *cpBody
	cpConstraintGetCollideBodies.cpBool   (*constraint.cpConstraint) As "_cpConstraintGetCollideBodies"
	cpConstraintGetErrorBias.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetErrorBias"
	cpConstraintGetImpulse.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetImpulse"
	cpConstraintGetMaxBias.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetMaxBias"
	cpConstraintGetMaxForce.cpFloat   (*constraint.cpConstraint) As "_cpConstraintGetMaxForce"
	cpConstraintGetPostSolveFunc.l  (*constraint.cpConstraint) As "_cpConstraintGetPostSolveFunc";  = cpConstraintPostSolveFunc
	cpConstraintGetPreSolveFunc.l   (*constraint.cpConstraint) As "_cpConstraintGetPreSolveFunc";  = cpConstraintPreSolveFunc
	cpConstraintGetSpace.l  (*constraint.cpConstraint) As "_cpConstraintGetSpace";  = *cpSpace
	cpConstraintGetUserData.l   (*constraint.cpConstraint) As "_cpConstraintGetUserData";  = *cpDataPointer
	cpConstraintInit  (*constraint.cpConstraint, *klass.cpConstraintClass, *a.cpBody, *b.cpBody) As "?cpConstraintInit@@YAXPAUcpConstraint@@PBUcpConstraintClass@@PAUcpBody@@2@Z"
	cpConstraintIsDampedRotarySpring.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsDampedRotarySpring"
	cpConstraintIsDampedSpring.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsDampedSpring"
	cpConstraintIsGearJoint.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsGearJoint"
	cpConstraintIsGrooveJoint.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsGrooveJoint"
	cpConstraintIsPinJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsPinJoint"
	cpConstraintIsPivotJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsPivotJoint"
	cpConstraintIsRatchetJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsRatchetJoint"
	cpConstraintIsRotaryLimitJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsRotaryLimitJoint"
	cpConstraintIsSimpleMotor.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsSimpleMotor"
	cpConstraintIsSlideJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsSlideJoint"
	cpConstraintSetCollideBodies  (*constraint.cpConstraint, collideBodies.cpBool) As "_cpConstraintSetCollideBodies"
	cpConstraintSetErrorBias  (*constraint.cpConstraint, errorBias.cpFloat) As "_cpConstraintSetErrorBias"
	cpConstraintSetMaxBias  (*constraint.cpConstraint, maxBias.cpFloat) As "_cpConstraintSetMaxBias"
	cpConstraintSetMaxForce   (*constraint.cpConstraint, maxForce.cpFloat) As "_cpConstraintSetMaxForce"
	cpConstraintSetPostSolveFunc  (*constraint.cpConstraint, *cpConstraintPostSolveFunc) As "_cpConstraintSetPostSolveFunc"
	cpConstraintSetPreSolveFunc   (*constraint.cpConstraint, *cpConstraintPreSolveFunc) As "_cpConstraintSetPreSolveFunc"
	cpConstraintSetUserData   (*constraint.cpConstraint, *cpDataPointer) As "_cpConstraintSetUserData"

	cpContactBufferGetArray.l   (*space.cpSpace) As "?cpContactBufferGetArray@@YAPAUcpContact@@PAUcpSpace@@@Z";  = *cpContact
	cpConvexHull.l  (count.l, *verts.cpVect, *result.cpVect, *first.Long, tol.cpFloat) As "_cpConvexHull"

	cpDampedRotarySpringAlloc.l   () As "_cpDampedRotarySpringAlloc";  = *cpDampedRotarySpring
	cpDampedRotarySpringGetDamping.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetDamping"
	cpDampedRotarySpringGetRestAngle.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetRestAngle"
	cpDampedRotarySpringGetSpringTorqueFunc.l   (*constraint.cpConstraint) As "_cpDampedRotarySpringGetSpringTorqueFunc";  = cpDampedRotarySpringTorqueFunc
	cpDampedRotarySpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetStiffness"
	cpDampedRotarySpringInit.l  (*spring.cpDampedRotarySpring, *a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "_cpDampedRotarySpringInit";  = *cpDampedRotarySpring
	cpDampedRotarySpringNew.l   (*a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "_cpDampedRotarySpringNew";  = *cpConstraint
	cpDampedRotarySpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "_cpDampedRotarySpringSetDamping"
	cpDampedRotarySpringSetRestAngle  (*constraint.cpConstraint, restAngle.cpFloat) As "_cpDampedRotarySpringSetRestAngle"
	cpDampedRotarySpringSetSpringTorqueFunc   (*constraint.cpConstraint, *cpDampedRotarySpringTorqueFunc) As "_cpDampedRotarySpringSetSpringTorqueFunc"
	cpDampedRotarySpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "_cpDampedRotarySpringSetStiffness"

	cpDampedSpringAlloc.l   () As "_cpDampedSpringAlloc";  = *cpDampedSpring
	cpDampedSpringGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringGetDamping.cpFloat  (*constraint.cpConstraint) As "_cpDampedSpringGetDamping"
	cpDampedSpringGetRestLength.cpFloat   (*constraint.cpConstraint) As "_cpDampedSpringGetRestLength"
	cpDampedSpringGetSpringForceFunc.l  (*constraint.cpConstraint) As "_cpDampedSpringGetSpringForceFunc";  = cpDampedSpringForceFunc
	cpDampedSpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "_cpDampedSpringGetStiffness"
	cpDampedSpringInit_pb.l   (*spring.cpDampedSpring, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringInit_pb@@YAPAUcpDampedSpring@@PAU1@PAUcpBody@@1PAUcpVect@@2NNN@Z";  = *cpDampedSpring
	cpDampedSpringNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1NNN@Z";  = *cpConstraint
	cpDampedSpringSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpDampedSpringSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpDampedSpringSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "_cpDampedSpringSetDamping"
	cpDampedSpringSetRestLength   (*constraint.cpConstraint, restLength.cpFloat) As "_cpDampedSpringSetRestLength"
	cpDampedSpringSetSpringForceFunc  (*constraint.cpConstraint, *cpDampedSpringForceFunc) As "_cpDampedSpringSetSpringForceFunc"
	cpDampedSpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "_cpDampedSpringSetStiffness"

	cpGearJointAlloc.l  () As "_cpGearJointAlloc";  = *cpGearJoint
	cpGearJointGetPhase.cpFloat   (*constraint.cpConstraint) As "_cpGearJointGetPhase"
	cpGearJointGetRatio.cpFloat   (*constraint.cpConstraint) As "_cpGearJointGetRatio"
	cpGearJointInit.l   (*joint.cpGearJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "_cpGearJointInit";  = *cpGearJoint
	cpGearJointNew.l  (*a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "_cpGearJointNew";  = *cpConstraint
	cpGearJointSetPhase   (*constraint.cpConstraint, phase.cpFloat) As "_cpGearJointSetPhase"
	cpGearJointSetRatio   (*constraint.cpConstraint, ratio.cpFloat) As "_cpGearJointSetRatio"

	cpGrooveJointAlloc.l  () As "_cpGrooveJointAlloc";  = *cpGrooveJoint
	cpGrooveJointGetAnchorB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointGetGrooveA_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointGetGrooveB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointInit_pb.l  (*joint.cpGrooveJoint, *a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointInit_pb@@YAPAUcpGrooveJoint@@PAU1@PAUcpBody@@1PAUcpVect@@22@Z";  = *cpGrooveJoint
	cpGrooveJointNew_pb.l   (*a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@11@Z";  = *cpConstraint
	cpGrooveJointSetAnchorB_pb  (*constraint.cpConstraint, *anchorB.cpVect) As "?cpGrooveJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointSetGrooveA_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointSetGrooveB_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"

	cpHashSetCount.l  (*set.cpHashSet) As "?cpHashSetCount@@YAHPAUcpHashSet@@@Z"
	cpHashSetEach   (*set.cpHashSet, *cpHashSetIteratorFunc, *data) As "?cpHashSetEach@@YAXPAUcpHashSet@@P6AXPAX1@Z1@Z"
	cpHashSetFilter   (*set.cpHashSet, *cpHashSetFilterFunc, *data) As "?cpHashSetFilter@@YAXPAUcpHashSet@@P6AEPAX1@Z1@Z"
	cpHashSetFind.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetFind@@YAPAXPAUcpHashSet@@IPAX@Z";  = *void
	cpHashSetFree   (*set.cpHashSet) As "?cpHashSetFree@@YAXPAUcpHashSet@@@Z"
	cpHashSetInsert.l   (*set.cpHashSet, hash.cpHashValue, *ptr, *cpHashSetTransFunc, *data) As "?cpHashSetInsert@@YAPAXPAUcpHashSet@@IPAXP6APAX11@Z1@Z";  = *void
	cpHashSetNew.l  (size.l, *cpHashSetEqlFunc) As "?cpHashSetNew@@YAPAUcpHashSet@@HP6AEPAX0@Z@Z";  = *cpHashSet
	cpHashSetRemove.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetRemove@@YAPAXPAUcpHashSet@@IPAX@Z";  = *void
	cpHashSetSetDefaultValue  (*set.cpHashSet, *default_value) As "?cpHashSetSetDefaultValue@@YAXPAUcpHashSet@@PAX@Z"

	cpLoopIndexes   (*verts.cpVect, count.l, *start.Long, *end.Long) As "?cpLoopIndexes@@YAXPBUcpVect@@HPAH1@Z"
; 	cpMessage   (*condition.Ascii, *file.Ascii, line.l, isError.l, isHardError.l, *message.Ascii, ...) As "_cpMessage"

	cpMomentForBox.cpFloat  (m.cpFloat, width.cpFloat, height.cpFloat) As "_cpMomentForBox"
	cpMomentForBox2.cpFloat   (m.cpFloat, *cpBB) As "_cpMomentForBox2"
	cpMomentForCircle_pb.cpFloat  (m.cpFloat, r1.cpFloat, r2.cpFloat, *offset.cpVect) As "?cpMomentForCircle_pb@@YANNNNPAUcpVect@@@Z"
	cpMomentForPoly_pb.cpFloat  (m.cpFloat, count.l, *verts.cpVect, *offset.cpVect, r.cpFloat) As "?cpMomentForPoly_pb@@YANNHPBUcpVect@@PAU1@N@Z"
	cpMomentForSegment_pb.cpFloat   (m.cpFloat, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpMomentForSegment_pb@@YANNPAUcpVect@@0N@Z"

	cpPinJointAlloc.l   () As "_cpPinJointAlloc";  = *cpPinJoint
	cpPinJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointGetDist.cpFloat   (*constraint.cpConstraint) As "_cpPinJointGetDist"
	cpPinJointInit_pb.l   (*joint.cpPinJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointInit_pb@@YAPAUcpPinJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2@Z";  = *cpPinJoint
	cpPinJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1@Z";  = *cpConstraint
	cpPinJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPinJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPinJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointSetDist   (*constraint.cpConstraint, dist.cpFloat) As "_cpPinJointSetDist"

	cpPivotJointAlloc.l   () As "_cpPivotJointAlloc";  = *cpPivotJoint
	cpPivotJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointInit_pb.l   (*joint.cpPivotJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointInit_pb@@YAPAUcpPivotJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2@Z";  = *cpPivotJoint
	cpPivotJointNew2_pb.l   (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointNew2_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1@Z";  = *cpConstraint
	cpPivotJointNew_pb.l  (*a.cpBody, *b.cpBody, *pivot.cpVect) As "?cpPivotJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@@Z";  = *cpConstraint
	cpPivotJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPivotJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPivotJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"

	cpPolyShapeAlloc.l  () As "_cpPolyShapeAlloc";  = *cpPolyShape
	cpPolyShapeGetCount.l   (*shape.cpShape) As "_cpPolyShapeGetCount"
	cpPolyShapeGetRadius.cpFloat  (*shape.cpShape) As "_cpPolyShapeGetRadius"
	cpPolyShapeGetVert_pb   (*shape.cpShape, i.l, *lp_out.cpVect) As "?cpPolyShapeGetVert_pb@@YAXPBUcpShape@@HPAUcpVect@@@Z"
	cpPolyShapeInit_pb.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeInit_pb@@YAPAUcpPolyShape@@PAU1@PAUcpBody@@HPBUcpVect@@PAUcpTransform@@N@Z";  = *cpPolyShape
	cpPolyShapeInitRaw.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "_cpPolyShapeInitRaw";  = *cpPolyShape
	cpPolyShapeNew_pb.l   (*body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@HPBUcpVect@@PAUcpTransform@@N@Z";  = *cpShape
	cpPolyShapeNewRaw.l   (*body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "_cpPolyShapeNewRaw";  = *cpShape
	cpPolyShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "_cpPolyShapeSetRadius"
	cpPolyShapeSetVerts_pb  (*shape.cpShape, count.l, *verts.cpVect, *transform.cpTransform) As "?cpPolyShapeSetVerts_pb@@YAXPAUcpShape@@HPAUcpVect@@PAUcpTransform@@@Z"
	cpPolyShapeSetVertsRaw  (*shape.cpShape, count.l, *verts.cpVect) As "_cpPolyShapeSetVertsRaw"

	cpRatchetJointAlloc.l   () As "_cpRatchetJointAlloc";  = *cpRatchetJoint
	cpRatchetJointGetAngle.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetAngle"
	cpRatchetJointGetPhase.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetPhase"
	cpRatchetJointGetRatchet.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetRatchet"
	cpRatchetJointInit.l  (*joint.cpRatchetJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "_cpRatchetJointInit";  = *cpRatchetJoint
	cpRatchetJointNew.l   (*a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "_cpRatchetJointNew";  = *cpConstraint
	cpRatchetJointSetAngle  (*constraint.cpConstraint, angle.cpFloat) As "_cpRatchetJointSetAngle"
	cpRatchetJointSetPhase  (*constraint.cpConstraint, phase.cpFloat) As "_cpRatchetJointSetPhase"
	cpRatchetJointSetRatchet  (*constraint.cpConstraint, ratchet.cpFloat) As "_cpRatchetJointSetRatchet"

	cpRotaryLimitJointAlloc.l   () As "_cpRotaryLimitJointAlloc";  = *cpRotaryLimitJoint
	cpRotaryLimitJointGetMax.cpFloat  (*constraint.cpConstraint) As "_cpRotaryLimitJointGetMax"
	cpRotaryLimitJointGetMin.cpFloat  (*constraint.cpConstraint) As "_cpRotaryLimitJointGetMin"
	cpRotaryLimitJointInit.l  (*joint.cpRotaryLimitJoint, *a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "_cpRotaryLimitJointInit";  = *cpRotaryLimitJoint
	cpRotaryLimitJointNew.l   (*a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "_cpRotaryLimitJointNew";  = *cpConstraint
	cpRotaryLimitJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "_cpRotaryLimitJointSetMax"
	cpRotaryLimitJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "_cpRotaryLimitJointSetMin"

	cpSegmentShapeAlloc.l   () As "_cpSegmentShapeAlloc";  = *cpSegmentShape
	cpSegmentShapeGetA_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetA_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetB_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetB_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetNormal_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetNormal_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetRadius.cpFloat   (*shape.cpShape) As "_cpSegmentShapeGetRadius"
	cpSegmentShapeInit_pb.l   (*seg.cpSegmentShape, *body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeInit_pb@@YAPAUcpSegmentShape@@PAU1@PAUcpBody@@PAUcpVect@@2N@Z";  = *cpSegmentShape
	cpSegmentShapeNew_pb.l  (*body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@PAUcpVect@@1N@Z";  = *cpShape
	cpSegmentShapeSetEndpoints_pb   (*shape.cpShape, *a.cpVect, *b.cpVect) As "?cpSegmentShapeSetEndpoints_pb@@YAXPAUcpShape@@PAUcpVect@@1@Z"
	cpSegmentShapeSetNeighbors_pb   (*shape.cpShape, *prev.cpVect, *next.cpVect) As "?cpSegmentShapeSetNeighbors_pb@@YAXPAUcpShape@@PAUcpVect@@1@Z"
	cpSegmentShapeSetRadius   (*shape.cpShape, radius.cpFloat) As "_cpSegmentShapeSetRadius"

	cpShapeCacheBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeCacheBB_pb@@YAXPAUcpShape@@PAUcpBB@@@Z"
	cpShapeDestroy  (*shape.cpShape) As "_cpShapeDestroy"
	cpShapeFree   (*shape.cpShape) As "_cpShapeFree"
	cpShapeGetArea.cpFloat  (*shape.cpShape) As "_cpShapeGetArea"
	cpShapeGetBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeGetBB_pb@@YAXPBUcpShape@@PAUcpBB@@@Z"
	cpShapeGetBody.l  (*shape.cpShape) As "_cpShapeGetBody";  = *cpBody
	cpShapeGetCenterOfGravity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetCenterOfGravity_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpShapeGetCollisionType.cpCollisionType   (*shape.cpShape) As "_cpShapeGetCollisionType"
	cpShapeGetDensity.cpFloat   (*shape.cpShape) As "_cpShapeGetDensity"
	cpShapeGetElasticity.cpFloat  (*shape.cpShape) As "_cpShapeGetElasticity"
	cpShapeGetFilter_pb   (*shape.cpShape, *lp_out.cpShapeFilter) As "?cpShapeGetFilter_pb@@YAXPBUcpShape@@PAUcpShapeFilter@@@Z"
	cpShapeGetFriction.cpFloat  (*shape.cpShape) As "_cpShapeGetFriction"
	cpShapeGetMass.cpFloat  (*shape.cpShape) As "_cpShapeGetMass"
	cpShapeGetMoment.cpFloat  (*shape.cpShape) As "_cpShapeGetMoment"
	cpShapeGetSensor.cpBool   (*shape.cpShape) As "_cpShapeGetSensor"
	cpShapeGetSpace.l   (*shape.cpShape) As "_cpShapeGetSpace";  = *cpSpace
	cpShapeGetSurfaceVelocity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetSurfaceVelocity_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpShapeGetUserData.l  (*shape.cpShape) As "_cpShapeGetUserData";  = *cpDataPointer
	cpShapeInit_pb.l  (*shape.cpShape, *klass.cpShapeClass, *body.cpBody, *massInfo.cpShapeMassInfo) As "?cpShapeInit_pb@@YAPAUcpShape@@PAU1@PBUcpShapeClass@@PAUcpBody@@PAUcpShapeMassInfo@@@Z";  = *cpShape
	cpShapePointQuery_pb.cpFloat  (*shape.cpShape, *p.cpVect, *info.cpPointQueryInfo) As "?cpShapePointQuery_pb@@YANPBUcpShape@@PAUcpVect@@PAUcpPointQueryInfo@@@Z"
	cpShapesCollide_pb  (*a.cpShape, *b.cpShape, *lp_out.cpContactPointSet) As "?cpShapesCollide_pb@@YAXPBUcpShape@@0PAUcpContactPointSet@@@Z"
	cpShapeSegmentQuery_pb.cpBool   (*shape.cpShape, *a.cpVect, *b.cpVect, radius.cpFloat, *info.cpSegmentQueryInfo) As "?cpShapeSegmentQuery_pb@@YAEPBUcpShape@@PAUcpVect@@1NPAUcpSegmentQueryInfo@@@Z"
	cpShapeSetBody  (*shape.cpShape, *body.cpBody) As "_cpShapeSetBody"
	cpShapeSetCollisionType   (*shape.cpShape, collisionType.cpCollisionType) As "_cpShapeSetCollisionType"
	cpShapeSetDensity   (*shape.cpShape, density.cpFloat) As "_cpShapeSetDensity"
	cpShapeSetElasticity  (*shape.cpShape, elasticity.cpFloat) As "_cpShapeSetElasticity"
	cpShapeSetFilter_pb   (*shape.cpShape, *filter.cpShapeFilter) As "?cpShapeSetFilter_pb@@YAXPAUcpShape@@PAUcpShapeFilter@@@Z"
	cpShapeSetFriction  (*shape.cpShape, friction.cpFloat) As "_cpShapeSetFriction"
	cpShapeSetMass  (*shape.cpShape, mass.cpFloat) As "_cpShapeSetMass"
	cpShapeSetSensor  (*shape.cpShape, sensor.cpBool) As "_cpShapeSetSensor"
	cpShapeSetSurfaceVelocity_pb  (*shape.cpShape, *surfaceVelocity.cpVect) As "?cpShapeSetSurfaceVelocity_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpShapeSetUserData  (*shape.cpShape, *cpDataPointer) As "_cpShapeSetUserData"
	cpShapeUpdate_pb  (*shape.cpShape, *transform.cpTransform, *lp_out.cpBB) As "?cpShapeUpdate_pb@@YAXPAUcpShape@@PAUcpTransform@@PAUcpBB@@@Z"
	cpShapeUpdateFunc   (*shape.cpShape, *unused) As "?cpShapeUpdateFunc@@YAXPAUcpShape@@PAX@Z"

	cpSimpleMotorAlloc.l  () As "_cpSimpleMotorAlloc";  = *cpSimpleMotor
	cpSimpleMotorGetRate.cpFloat  (*constraint.cpConstraint) As "_cpSimpleMotorGetRate"
	cpSimpleMotorInit.l   (*joint.cpSimpleMotor, *a.cpBody, *b.cpBody, rate.cpFloat) As "_cpSimpleMotorInit";  = *cpSimpleMotor
	cpSimpleMotorNew.l  (*a.cpBody, *b.cpBody, rate.cpFloat) As "_cpSimpleMotorNew";  = *cpConstraint
	cpSimpleMotorSetRate  (*constraint.cpConstraint, rate.cpFloat) As "_cpSimpleMotorSetRate"

	cpSlideJointAlloc.l   () As "_cpSlideJointAlloc";  = *cpSlideJoint
	cpSlideJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointGetMax.cpFloat  (*constraint.cpConstraint) As "_cpSlideJointGetMax"
	cpSlideJointGetMin.cpFloat  (*constraint.cpConstraint) As "_cpSlideJointGetMin"
	cpSlideJointInit_pb.l   (*joint.cpSlideJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointInit_pb@@YAPAUcpSlideJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2NN@Z";  = *cpSlideJoint
	cpSlideJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1NN@Z";  = *cpConstraint
	cpSlideJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpSlideJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpSlideJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "_cpSlideJointSetMax"
	cpSlideJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "_cpSlideJointSetMin"

	cpSpaceActivateBody   (*space.cpSpace, *body.cpBody) As "?cpSpaceActivateBody@@YAXPAUcpSpace@@PAUcpBody@@@Z"
	cpSpaceAddBody.l  (*space.cpSpace, *body.cpBody) As "_cpSpaceAddBody";  = *cpBody
	cpSpaceAddCollisionHandler.l  (*space.cpSpace, a.cpCollisionType, b.cpCollisionType) As "_cpSpaceAddCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddConstraint.l  (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceAddConstraint";  = *cpConstraint
	cpSpaceAddDefaultCollisionHandler.l   (*space.cpSpace) As "_cpSpaceAddDefaultCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddPostStepCallback.cpBool   (*space.cpSpace, *cpPostStepFunc, *key, *data) As "_cpSpaceAddPostStepCallback"
	cpSpaceAddShape.l   (*space.cpSpace, *shape.cpShape) As "_cpSpaceAddShape";  = *cpShape
	cpSpaceAddWildcardHandler.l   (*space.cpSpace, type.cpCollisionType) As "_cpSpaceAddWildcardHandler";  = *cpCollisionHandler
	cpSpaceAlloc.l  () As "_cpSpaceAlloc";  = *cpSpace
	cpSpaceArbiterSetFilter.cpBool  (*arb.cpArbiter, *space.cpSpace) As "?cpSpaceArbiterSetFilter@@YAEPAUcpArbiter@@PAUcpSpace@@@Z"
	cpSpaceBBQuery_pb   (*space.cpSpace, *bb.cpBB, *filter.cpShapeFilter, *cpSpaceBBQueryFunc, *data) As "?cpSpaceBBQuery_pb@@YAXPAUcpSpace@@PAUcpBB@@PAUcpShapeFilter@@P6AXPAUcpShape@@PAX@Z4@Z"
	cpSpaceCollideShapes.cpCollisionID  (*a.cpShape, *b.cpShape, id.cpCollisionID, *space.cpSpace) As "?cpSpaceCollideShapes@@YAIPAUcpShape@@0IPAUcpSpace@@@Z"
	cpSpaceContainsBody.cpBool  (*space.cpSpace, *body.cpBody) As "_cpSpaceContainsBody"
	cpSpaceContainsConstraint.cpBool  (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceContainsConstraint"
	cpSpaceContainsShape.cpBool   (*space.cpSpace, *shape.cpShape) As "_cpSpaceContainsShape"
	cpSpaceDebugDraw  (*space.cpSpace, *options.cpSpaceDebugDrawOptions) As "_cpSpaceDebugDraw"
	cpSpaceDestroy  (*space.cpSpace) As "_cpSpaceDestroy"
	cpSpaceEachBody   (*space.cpSpace, *cpSpaceBodyIteratorFunc, *data) As "_cpSpaceEachBody"
	cpSpaceEachConstraint   (*space.cpSpace, *cpSpaceConstraintIteratorFunc, *data) As "_cpSpaceEachConstraint"
	cpSpaceEachShape  (*space.cpSpace, *cpSpaceShapeIteratorFunc, *data) As "_cpSpaceEachShape"
	cpSpaceFilterArbiters   (*space.cpSpace, *body.cpBody, *filter.cpShape) As "?cpSpaceFilterArbiters@@YAXPAUcpSpace@@PAUcpBody@@PAUcpShape@@@Z"
	cpSpaceFree   (*space.cpSpace) As "_cpSpaceFree"
	cpSpaceGetCollisionBias.cpFloat   (*space.cpSpace) As "_cpSpaceGetCollisionBias"
	cpSpaceGetCollisionPersistence.cpTimestamp  (*space.cpSpace) As "_cpSpaceGetCollisionPersistence"
	cpSpaceGetCollisionSlop.cpFloat   (*space.cpSpace) As "_cpSpaceGetCollisionSlop"
	cpSpaceGetCurrentTimeStep.cpFloat   (*space.cpSpace) As "_cpSpaceGetCurrentTimeStep"
	cpSpaceGetDamping.cpFloat   (*space.cpSpace) As "_cpSpaceGetDamping"
	cpSpaceGetGravity_pb  (*space.cpSpace, *lp_out.cpVect) As "?cpSpaceGetGravity_pb@@YAXPBUcpSpace@@PAUcpVect@@@Z"
	cpSpaceGetIdleSpeedThreshold.cpFloat  (*space.cpSpace) As "_cpSpaceGetIdleSpeedThreshold"
	cpSpaceGetIterations.l  (*space.cpSpace) As "_cpSpaceGetIterations"
	cpSpaceGetPostStepCallback.l  (*space.cpSpace, *key) As "?cpSpaceGetPostStepCallback@@YAPAUcpPostStepCallback@@PAUcpSpace@@PAX@Z";  = *cpPostStepCallback
	cpSpaceGetSleepTimeThreshold.cpFloat  (*space.cpSpace) As "_cpSpaceGetSleepTimeThreshold"
	cpSpaceGetStaticBody.l  (*space.cpSpace) As "_cpSpaceGetStaticBody";  = *cpBody
	cpSpaceGetUserData.l  (*space.cpSpace) As "_cpSpaceGetUserData";  = *cpDataPointer
	cpSpaceHashAlloc.l  () As "_cpSpaceHashAlloc";  = *cpSpaceHash
	cpSpaceHashInit.l   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSpaceHashInit";  = *cpSpatialIndex
	cpSpaceHashNew.l  (celldim.cpFloat, cells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSpaceHashNew";  = *cpSpatialIndex
	cpSpaceHashResize   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l) As "_cpSpaceHashResize"
	cpSpaceInit.l   (*space.cpSpace) As "_cpSpaceInit";  = *cpSpace
	cpSpaceIsLocked.cpBool  (*space.cpSpace) As "_cpSpaceIsLocked"
	cpSpaceLock   (*space.cpSpace) As "?cpSpaceLock@@YAXPAUcpSpace@@@Z"
	cpSpaceNew.l  () As "_cpSpaceNew";  = *cpSpace
	cpSpacePointQuery_pb  (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *cpSpacePointQueryFunc, *data) As "?cpSpacePointQuery_pb@@YAXPAUcpSpace@@PAUcpVect@@NPAUcpShapeFilter@@P6AXPAUcpShape@@U2@N4PAX@Z5@Z"
	cpSpacePointQueryNearest_pb.l   (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *out.cpPointQueryInfo) As "?cpSpacePointQueryNearest_pb@@YAPAUcpShape@@PAUcpSpace@@PAUcpVect@@NPAUcpShapeFilter@@PAUcpPointQueryInfo@@@Z";  = *cpShape
	cpSpaceProcessComponents  (*space.cpSpace, dt.cpFloat) As "?cpSpaceProcessComponents@@YAXPAUcpSpace@@N@Z"
	cpSpacePushContacts   (*space.cpSpace, count.l) As "?cpSpacePushContacts@@YAXPAUcpSpace@@H@Z"
	cpSpacePushFreshContactBuffer   (*space.cpSpace) As "?cpSpacePushFreshContactBuffer@@YAXPAUcpSpace@@@Z"
	cpSpaceReindexShape   (*space.cpSpace, *shape.cpShape) As "_cpSpaceReindexShape"
	cpSpaceReindexShapesForBody   (*space.cpSpace, *body.cpBody) As "_cpSpaceReindexShapesForBody"
	cpSpaceReindexStatic  (*space.cpSpace) As "_cpSpaceReindexStatic"
	cpSpaceRemoveBody   (*space.cpSpace, *body.cpBody) As "_cpSpaceRemoveBody"
	cpSpaceRemoveConstraint   (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceRemoveConstraint"
	cpSpaceRemoveShape  (*space.cpSpace, *shape.cpShape) As "_cpSpaceRemoveShape"
	cpSpaceSegmentQuery_pb  (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *cpSpaceSegmentQueryFunc, *data) As "?cpSpaceSegmentQuery_pb@@YAXPAUcpSpace@@PAUcpVect@@1NPAUcpShapeFilter@@P6AXPAUcpShape@@U2@4NPAX@Z5@Z"
	cpSpaceSegmentQueryFirst_pb.l   (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *out.cpSegmentQueryInfo) As "?cpSpaceSegmentQueryFirst_pb@@YAPAUcpShape@@PAUcpSpace@@PAUcpVect@@1NPAUcpShapeFilter@@PAUcpSegmentQueryInfo@@@Z";  = *cpShape
	cpSpaceSetCollisionBias   (*space.cpSpace, collisionBias.cpFloat) As "_cpSpaceSetCollisionBias"
	cpSpaceSetCollisionPersistence  (*space.cpSpace, collisionPersistence.cpTimestamp) As "_cpSpaceSetCollisionPersistence"
	cpSpaceSetCollisionSlop   (*space.cpSpace, collisionSlop.cpFloat) As "_cpSpaceSetCollisionSlop"
	cpSpaceSetDamping   (*space.cpSpace, damping.cpFloat) As "_cpSpaceSetDamping"
	cpSpaceSetGravity_pb  (*space.cpSpace, *gravity.cpVect) As "?cpSpaceSetGravity_pb@@YAXPAUcpSpace@@PAUcpVect@@@Z"
	cpSpaceSetIdleSpeedThreshold  (*space.cpSpace, idleSpeedThreshold.cpFloat) As "_cpSpaceSetIdleSpeedThreshold"
	cpSpaceSetIterations  (*space.cpSpace, iterations.l) As "_cpSpaceSetIterations"
	cpSpaceSetSleepTimeThreshold  (*space.cpSpace, sleepTimeThreshold.cpFloat) As "_cpSpaceSetSleepTimeThreshold"
	cpSpaceSetStaticBody  (*space.cpSpace, *body.cpBody) As "?cpSpaceSetStaticBody@@YAXPAUcpSpace@@PAUcpBody@@@Z"
	cpSpaceSetUserData  (*space.cpSpace, *cpDataPointer) As "_cpSpaceSetUserData"
	cpSpaceShapeQuery.cpBool  (*space.cpSpace, *shape.cpShape, *cpSpaceShapeQueryFunc, *data) As "_cpSpaceShapeQuery"
	cpSpaceStep   (*space.cpSpace, dt.cpFloat) As "_cpSpaceStep"
	cpSpaceUnlock   (*space.cpSpace, runPostStep.cpBool) As "?cpSpaceUnlock@@YAXPAUcpSpace@@E@Z"
	cpSpaceUseSpatialHash   (*space.cpSpace, cDim.cpFloat, count.l) As "_cpSpaceUseSpatialHash"

	cpSpatialIndexCollideStatic   (*dynamicIndex.cpSpatialIndex, *staticIndex.cpSpatialIndex, *cpSpatialIndexQueryFunc, *data) As "_cpSpatialIndexCollideStatic"
	cpSpatialIndexFree  (*index.cpSpatialIndex) As "_cpSpatialIndexFree"
	cpSpatialIndexInit.l  (*index.cpSpatialIndex, *klass.cpSpatialIndexClass, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "?cpSpatialIndexInit@@YAPAUcpSpatialIndex@@PAU1@PAUcpSpatialIndexClass@@P6A?AUcpBB@@PAX@Z0@Z";  = *cpSpatialIndex

	cpSweep1DAlloc.l  () As "_cpSweep1DAlloc";  = *cpSweep1D
	cpSweep1DInit.l   (*sweep.cpSweep1D, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSweep1DInit";  = *cpSpatialIndex
	cpSweep1DNew.l  (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSweep1DNew";  = *cpSpatialIndex
EndImport

; ========================================================================================================

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 4
; Folding = -XFAAAAAAAA-
; Markers = 211,1203
; EnableUnicode
; EnableXP
; CPU = 1
; CompileSourceDirectory
; EnableCompileCount = 36
; EnableBuildCount = 0