﻿EnableExplicit
	XIncludeFile "ChipMunk2D.pbi"
;==========================================================================================================================-
; This is simple "helloword" ported from chipmunk documentation
; It creates a ball which falls on a surface and rolls a bit, displaying all data in debug
;====================================================================================================-

Define *space.cpSpace = cpSpaceNew() ; Create an empty space.
Define gravity.cpVect : cpv(gravity, 0.0, -100) ; cpVect is a 2D vector And cpv() is a shortcut For initializing them.
	cpSpaceSetGravity_pb(*space, gravity)	

 
; Add a Static line segment shape For the ground.
; We'll make it slightly tilted so the ball will roll off.
; We attach it To space->staticBody To tell Chipmunk it shouldn't be movable.
Define A.cpVect : cpv(A, -20, 5)
Define B.cpVect : cpv(B, 20, 4.5)
Define *ground.cpShape = cpSegmentShapeNew_pb(*space\staticBody, A, B, 0)
	cpShapeSetFriction(*ground, 1)
	cpSpaceAddShape(*space, *ground)


; Now let's make a ball that falls onto the line and rolls off.
; First we need To make a cpBody To hold the physical Properties of the object.
; These include the mass, position, velocity, angle, etc. of the object.
; Then we attach collision shapes To the cpBody To give it a size And shape.

Define radius.cpFloat = 5
Define mass.cpFloat = 1

; The moment of inertia is like mass For rotation
; Use the cpMomentFor*() functions To help you approximate it.
Define moment.cpFloat = cpMomentForCircle_pb(mass, 0, radius, cpvzero)


; The cpSpaceAdd*() functions Return the thing that you are adding.
; It's convenient to create and add an object in one line.
Define *ballBody.cpBody = cpSpaceAddBody(*space, cpBodyNew(mass, moment))

Define bodyPos.cpVect : cpv(bodyPos, 0, 50)
	cpBodySetPosition_pb(*ballBody, bodyPos)
	
; Now we create the collision shape For the ball.
; You can create multiple collision shapes that point To the same body.
; They will all be attached To the body And move around To follow it.
Define *ballShape.cpShape = cpSpaceAddShape(*space, cpCircleShapeNew_pb(*ballBody, radius, cpvzero))
	cpShapeSetFriction(*ballShape, 0.7)

; Now that it's all set up, we simulate all the objects in the space by
; stepping forward through time in small increments called steps.
; It is *highly* recommended To use a fixed size time Step.
Define timeStep.cpFloat = 1.0/60.0
Define Time.cpFloat

Define pos.cpVect, vel.cpVect
While Time < 2 : Time + timeStep
	cpBodyGetPosition_pb(*ballBody, pos)
	cpBodyGetVelocity_pb(*ballBody, vel)

	Debug "Time is " + StrF(time,2) + 
		". ballBody is at (" + StrF(pos\x, 2) + "," + StrF(pos\y, 2) + ". It's velocity is (" + StrF(vel\x, 2) + "," + StrF(vel\y, 2) + ")"

	cpSpaceStep(*space, timeStep);
Wend

; Clean up our objects And exit!
; You should always use clear functions to all objects cause ChipMunk doesn't have garbage collector [like PB :)]
cpShapeFree(*ballShape)
cpBodyFree(*ballBody)
cpShapeFree(*ground)
cpSpaceFree(*space)

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 22
; FirstLine = 22
; Folding = -
; EnableUnicode
; EnableXP
; CPU = 1
; CompileSourceDirectory
; EnableCompileCount = 2
; EnableBuildCount = 0